@@ -201,18 +201,24 @@ class IndexedArray(awkward.array.base.AwkwardArrayWithContent):
 
         return getattr(ufunc, method)(*inputs, **kwargs)
 
-    @classmethod
-    def concat(cls, first, *rest):
-        raise NotImplementedError
+    def _reduce(self, ufunc, identity, dtype, regularaxis):
+        if awkward.util._hasjagged(self._content):
+            return self.copy(content=self._content._reduce(ufunc, identity, dtype, regularaxis))
 
-    def pandas(self):
-        import pandas
-        self._valid()
+        elif isinstance(self._content, awkward.array.table.Table):
+            out = awkward.array.table.Table()
+            for n, x in self._content._contents.items():
+                out[n] = self.copy(content=x)
+            return out._reduce(ufunc, identity, dtype, regularaxis)
 
-        if isinstance(self._content, awkward.util.numpy.ndarray):
-            return pandas.DataFrame(self._content[self._index])
         else:
-            return self._content[self._index].pandas()
+            return ufunc.reduce(self._prepare(identity, dtype))
+
+    def _prepare(self, identity, dtype):
+        if isinstance(self._content, awkward.util.numpy.ndarray):
+            return self._content[self._index]
+        else:
+            return self._content._prepare(identity, dtype)[self._index]
 
 class SparseArray(awkward.array.base.AwkwardArrayWithContent):
     """
@@ -586,9 +592,21 @@ class SparseArray(awkward.array.base.AwkwardArrayWithContent):
 
         return getattr(ufunc, method)(*inputs, **kwargs)
 
-    @classmethod
-    def concat(cls, first, *rest):
-        raise NotImplementedError
+    def _reduce(self, ufunc, identity, dtype, regularaxis):
+        if awkward.util._hasjagged(self._content):
+            return self.copy(content=self._content._reduce(ufunc, identity, dtype, regularaxis))
 
-    def pandas(self):
-        raise NotImplementedError
+        elif isinstance(self._content, awkward.array.table.Table):
+            out = awkward.array.table.Table()
+            for n, x in self._content._contents.items():
+                out[n] = self.copy(content=x)
+            return out._reduce(ufunc, identity, dtype, regularaxis)
+
+        else:
+            return ufunc.reduce(self._prepare(identity, dtype))
+
+    def _prepare(self, identity, dtype):
+        if isinstance(self._content, awkward.util.numpy.ndarray):
+            return self.dense
+        else:
+            return self.copy(content=self._content._prepare(identity, dtype)).dense
