@@ -28,6 +28,7 @@
 # OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 # OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
+import math
 import numbers
 
 import awkward.util
@@ -163,7 +164,7 @@ class Type(object):
                 for n, y in x._fields.items():
                     if first is None:
                         first = y._takes
-                    elif first != y._takes:
+                    elif first != y._takes or math.isinf(y._takes):
                         break
                     newtable._fields[n] = y._to
                 else:
@@ -582,8 +583,12 @@ class OptionType(Type):
             return False
         else:
             seen.add(id(self))
-            if isinstance(other, OptionType) and self._type._eq(other._type, seen, ignoremask=ignoremask):
-                return True
+            if isinstance(other, OptionType):
+                if isinstance(self._type, Type) and self._type._eq(other._type, seen, ignoremask=ignoremask):
+                    return True
+                elif not isinstance(self._type, Type) and self._type == other._type:
+                    return True
+
             if ignoremask:    # applied asymmetrically; only the left can ignore mask
                 if isinstance(self._type, Type):
                     return self._type._eq(other, seen, ignoremask=ignoremask)
